@@ -1,5 +1,5 @@
 /*
- * qstat 2.8
+ * qstat 2.14
  * by Steve Jankowski
  *
  * New Half-Life2 query protocol
@@ -12,8 +12,8 @@
 
 #include "qserver.h"
 
-void send_a2s_request_packet(struct qserver *server);
-void send_a2s_rule_request_packet(struct qserver *server);
-void deal_with_a2s_packet(struct qserver *server, char *rawpkt, int pktlen);
+query_status_t send_a2s_request_packet(struct qserver *server);
+query_status_t send_a2s_rule_request_packet(struct qserver *server);
+query_status_t deal_with_a2s_packet(struct qserver *server, char *rawpkt, int pktlen);
 
 #endif
