@@ -1,5 +1,5 @@
 /*
- * qstat 2.6
+ * qstat 2.14
  * by Steve Jankowski
  * steve@qstat.org
  * http://www.qstat.org
@@ -39,16 +39,16 @@ static int n_config_types;
 static int max_config_types;
 static int next_gametype_id= LAST_BUILTIN_SERVER+1;
 
-static int load_config_file( char *filename);
-static int try_load_config_file( char *filename, int show_error);
+static int load_config_file(char const* filename);
+static int try_load_config_file(char const* filename, int const show_error);
 static int pf_top_level( char *text, void *context);
 static int pf_gametype_new( char *text, void *context);
 static int pf_gametype_modify( char *text, void *context);
 
 static int set_game_type_value( server_type *gametype, int key, char *value);
 static int modify_game_type_value( server_type *gametype, int key, char *value);
-static int set_packet_value( server_type *gametype, char *value,
-	char *packet_name, char **packet, int *len);
+static int set_packet_value(server_type* gametype, char const* value,
+	char const* packet_name, char** packet, int* len);
 
 static char *first_token( char *text);
 static char *next_token();
@@ -60,14 +60,14 @@ static unsigned int parse_oct( char *oct, int len, int *error);
 static char *force_lower_case( char *string);
 static char *force_upper_case( char *string);
 static char *get_token();
-static void * memdup( void *mem, unsigned int len);
+static void *memdup( const void *mem, unsigned int len);
 
 int parse_config_file( FILE *file);
 
 static int (*parse_func)( char *, void *);
 static void *parse_context;
 static int line;
-static char *current_file_name;
+static char const* current_file_name;
 
 static char *parse_text;
 static char *parse_end;
@@ -103,7 +103,7 @@ typedef struct _config_key {
     char *key_name;
 } ConfigKey;
 
-static ConfigKey new_keys[] = {
+static ConfigKey const new_keys[] = {
 { CK_MASTER_PROTOCOL, "master protocol" },
 { CK_MASTER_QUERY, "master query" },
 { CK_MASTER_PACKET, "master packet" },
@@ -121,7 +121,7 @@ static ConfigKey new_keys[] = {
 { 0, NULL },
 };
 
-static ConfigKey modify_keys[] = {
+static ConfigKey const modify_keys[] = {
 { CK_MASTER_PROTOCOL, "master protocol" },
 { CK_MASTER_QUERY, "master query" },
 { CK_MASTER_PACKET, "master packet" },
@@ -136,7 +136,7 @@ typedef struct {
 } ServerFlag;
 
 #define SERVER_FLAG(x) { #x, x }
-ServerFlag server_flags[] = {
+ServerFlag const server_flags[] = {
     SERVER_FLAG(TF_SINGLE_QUERY),
     SERVER_FLAG(TF_OUTFILE),
     SERVER_FLAG(TF_MASTER_MULTI_RESPONSE),
@@ -156,7 +156,7 @@ ServerFlag server_flags[] = {
 };
 #undef SERVER_FLAG
 
-static int get_config_key( char *first_token, ConfigKey *keys);
+static int get_config_key( char *first_token, const ConfigKey *keys);
 static void add_config_type( server_type *gametype);
 static server_type * get_config_type( char *game_type);
 static void copy_server_type( server_type *dest, server_type *source);
@@ -239,7 +239,7 @@ qsc_load_default_config_files()
     filename= getenv( "HOME");
     if ( filename != NULL && filename[0] != '\0')  {
 	char path[1024];
-	sprintf( path, "%s/%s", filename, HOME_CONFIG_FILE);
+	snprintf( path, (sizeof(path) -1),"%s/%s", var, HOME_CONFIG_FILE);
     }
 /* 1. $QSTAT_CONFIG
    2. UNIX: $HOME/.qstatrc         WIN: $HOME/qstat.cfg
@@ -254,7 +254,7 @@ qsc_load_default_config_files()
 }
 
 int
-qsc_load_config_file( char *filename)
+qsc_load_config_file(char const* filename)
 {
     int rc= load_config_file( filename);
     if ( rc == -2)  {
@@ -275,7 +275,7 @@ qsc_get_config_server_types( int *n_config_types_ref)
 }
 
 STATIC int
-try_load_config_file( char *filename, int show_error)
+try_load_config_file(char const* filename, int const show_error)
 {
     int rc= load_config_file( filename);
     if ( rc == -2 && show_error)  {
@@ -290,7 +290,7 @@ try_load_config_file( char *filename, int show_error)
 }
 
 STATIC int
-load_config_file( char *filename)
+load_config_file(char const* filename)
 {
     FILE *file;
     int rc;
@@ -522,7 +522,7 @@ pf_gametype_new( char *text, void *_context)
 }
 
 STATIC int
-get_config_key( char *first_token, ConfigKey *keys)
+get_config_key( char *first_token, const ConfigKey *keys)
 {
     char key_name[1024], *token;
     int key= 0;
@@ -729,7 +729,7 @@ set_game_type_value( server_type *gametype, int key, char *value)
 }
 
 STATIC int
-set_packet_value( server_type *gametype, char *value, char *packet_name,
+set_packet_value(server_type* gametype, char const* value, char const* packet_name,
 	char **packet, int *len)
 {
     if ( gametype->master)  {
@@ -740,10 +740,12 @@ set_packet_value( server_type *gametype, char *value, char *packet_name,
 	    REPORT_ERROR((stderr, "Empty %s packet", packet_name));
 	    return -1;
 	}
-	if ( *packet == NULL)  {
-	    REPORT_ERROR((stderr, "Cannot set %s packet; extend game type does not define a %s packet", packet_name, packet_name));
-	    return -1;
-	}
+	// Removed as this doesn't seem to be any reason why we can't do this and it works just fine for warsow
+	//if ( *packet == NULL)  {
+	//    REPORT_ERROR((stderr, "Cannot set %s packet; extend game type does not define a %s packet", packet_name, packet_name));
+	//    return -1;
+	//}
+
 	*packet= memdup( value, value_len);
 	*len= value_len;
     return 0;
@@ -881,10 +883,15 @@ copy_server_type( server_type *dest, server_type *source)
 }
 
 STATIC void *
-memdup( void *mem, unsigned int len)
+memdup( const void *mem, unsigned int len)
 {
     void *result= malloc( len);
-    memcpy( result, mem, len);
+	if ( NULL == result )
+	{
+	    REPORT_ERROR(( stderr, "Failed to malloc %d bytes of memory", len ) );
+		return NULL;
+	}
+	memcpy( result, mem, len);
     return result;
 }
 
