@@ -1,5 +1,5 @@
 /*
- * qstat 2.6
+ * qstat 2.14
  * by Steve Jankowski
  *
  * debug helper functions
@@ -8,13 +8,13 @@
  * Licensed under the Artistic License, see LICENSE.txt for license terms
  */
 
-#include "debug.h"
-
 #include <stdio.h>
 #include <stdlib.h>
 #ifndef _WIN32
 #include <unistd.h>
 #include <sys/param.h>
+#else
+#include <stdarg.h>
 #endif
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -24,15 +24,26 @@
 #include <fcntl.h>
 #include <stdarg.h>
 
+#define QSTAT_DEBUG_C
+#include "debug.h"
+
+#ifdef ENABLE_DUMP
+#ifndef _WIN32
+	#include <sys/socket.h>
+#else
+	#include <winsock.h>
+#endif
+#endif
+
 #ifdef DEBUG
+
 void _debug(const char* file, int line, const char* function, int level, const char* fmt, ...)
 {
 	va_list ap;
 	char buf[9];
-	time_t now;
+	time_t now = time( NULL );
 
-	now = time(NULL);
-	strftime(buf,9,"%T",localtime(&now));
+	strftime(buf,9,"%H:%M:%S", localtime( &now ) );
 
 	fprintf(stderr, "debug(%d) %s %s:%d %s() - ", level, buf, file, line, function);
 
@@ -90,24 +101,42 @@ void malformed_packet(const struct qserver* server, const char* fmt, ...)
 
 #ifdef ENABLE_DUMP
 static unsigned count = 0;
-#endif
-void dump_packet(const char* buf, int buflen)
+static void _dump_packet(const char* tag, const char* buf, int buflen)
 {
-#ifdef ENABLE_DUMP
 	char fn[PATH_MAX] = {0};
 	int fd;
-	sprintf(fn, "dump%03u", count++);
+	sprintf(fn, "%03u_%s.pkt", count++, tag);
 	fprintf(stderr, "dumping to %s\n", fn);
 	fd = open(fn, O_WRONLY|O_CREAT|O_EXCL, 0644);
 	if(fd == -1) { perror("open"); return; }
 	if(write(fd, buf, buflen) == -1)
 		perror("write");
 	close(fd);
+}
+
+ssize_t send_dump(int s, const void *buf, size_t len, int flags)
+{
+	if(do_dump)
+		_dump_packet("send", buf, len);
+	return send(s, buf, len, flags);
+}
+#endif
+
+void dump_packet(const char* buf, int buflen)
+{
+#ifdef ENABLE_DUMP
+	_dump_packet("recv", buf, buflen);
 #endif
 }
 
 void
-print_packet( struct qserver *server, const char *buf, int buflen)
+print_packet( struct qserver *server, const char *buf, int buflen )
+{
+	output_packet( server, buf, buflen, 0 );
+}
+
+void
+output_packet( struct qserver *server, const char *buf, int buflen, int to )
 {
 	static char *hex= "0123456789abcdef";
 	unsigned char *p= (unsigned char*)buf;
@@ -115,27 +144,40 @@ print_packet( struct qserver *server, const char *buf, int buflen)
 	char line[256];
 
 	if ( server != NULL)
-	fprintf( stderr, "FROM %s len %d\n", server->arg, buflen );
-
-	for ( i= buflen; i ; offset+= 16)  {
-	memset( line, ' ', 256);
-	h= 0;
-	h+= sprintf( line, "%5d:", offset);
-	a= astart = h + 16*2 + 16/4 + 2;
-	for ( b=16; b && i; b--, i--, p++)  {
-		if ( (b & 3) == 0)
-		line[h++]= ' ';
-		line[h++]= hex[*p >> 4];
-		line[h++]= hex[*p & 0xf];
-		if ( isprint( *p))
-		line[a++]= *p;
-		else
-		line[a++]= '.';
-		if((a-astart)==8) line[a++] = ' ';
+	{
+		fprintf( stderr, "%s %s len %d\n", ( to ) ? "TO" : "FROM", server->arg, buflen );
 	}
-	line[a]= '\0';
-	fputs( line, stderr);
-	fputs( "\n", stderr);
+
+	for ( i= buflen; i ; offset+= 16)
+	{
+		memset( line, ' ', 256);
+		h= 0;
+		h+= sprintf( line, "%5d:", offset);
+		a= astart = h + 16*2 + 16/4 + 2;
+		for ( b=16; b && i; b--, i--, p++)
+		{
+			if ( (b & 3) == 0)
+			{
+				line[h++]= ' ';
+			}
+			line[h++]= hex[*p >> 4];
+			line[h++]= hex[*p & 0xf];
+			if ( isprint( *p))
+			{
+				line[a++]= *p;
+			}
+			else
+			{
+				line[a++]= '.';
+			}
+			if((a-astart)==8)
+			{
+				line[a++] = ' ';
+			}
+		}
+		line[a]= '\0';
+		fputs( line, stderr);
+		fputs( "\n", stderr);
 	}
 	fputs( "\n", stderr);
 }
