@@ -1,5 +1,5 @@
 /*
- * qstat 2.8
+ * qstat 2.14
  * by Steve Jankowski
  *
  * New Gamespy v2 query protocol
@@ -20,7 +20,7 @@
 #include "qstat.h"
 #include "packet_manip.h"
 
-void send_gs2_request_packet( struct qserver *server )
+query_status_t send_gs2_request_packet( struct qserver *server )
 {
 	// The below should work but seems to make no difference to what some
 	// servers send
@@ -35,13 +35,13 @@ void send_gs2_request_packet( struct qserver *server )
 		server->type->status_packet[9] = 0x00;
 	}
 
-	send_packet( server, server->type->status_packet, server->type->status_len );
+	return send_packet( server, server->type->status_packet, server->type->status_len );
 }
 
 
 // See the following for protocol details:
 // http://dev.kquery.com/index.php?article=42
-void deal_with_gs2_packet( struct qserver *server, char *rawpkt, int pktlen )
+query_status_t deal_with_gs2_packet( struct qserver *server, char *rawpkt, int pktlen )
 {
 	char *ptr = rawpkt;
 	char *end = rawpkt + pktlen;
@@ -57,8 +57,7 @@ void deal_with_gs2_packet( struct qserver *server, char *rawpkt, int pktlen )
 	if ( pktlen < 15 )
 	{
 		// invalid packet?
-		cleanup_qserver( server, 1);
-		return;
+		return PKT_ERROR;
 	}
 
 	server->n_servers++;
@@ -95,8 +94,7 @@ void deal_with_gs2_packet( struct qserver *server, char *rawpkt, int pktlen )
 			{
 				malformed_packet( server, "no player headers" );
 			}
-			cleanup_qserver( server, 1);
-			return;
+			return PKT_ERROR;
 		}
 		ptr += var_len + 1;
 
@@ -125,6 +123,11 @@ void deal_with_gs2_packet( struct qserver *server, char *rawpkt, int pktlen )
 		{
 			server->map_name = strdup( val );
 		}
+		else if( 0 == strcmp( var, "map" ) )
+		{
+			// For BF2MC compatibility
+			server->map_name = strdup( val );
+		}
 		else if( 0 == strcmp( var, "maxplayers" ) )
 		{
 			server->max_players = atoi( val );
@@ -154,8 +157,7 @@ void deal_with_gs2_packet( struct qserver *server, char *rawpkt, int pktlen )
 		// no more info should be player headers here as we
 		// requested it
 		malformed_packet( server, "no player headers" );
-		cleanup_qserver( server, 1);
-		return;
+		return PKT_ERROR;
 	}
 
 	// player info header
@@ -169,8 +171,7 @@ void deal_with_gs2_packet( struct qserver *server, char *rawpkt, int pktlen )
 	if ( ptr >= end )
 	{
 		malformed_packet( server, "no player headers" );
-		cleanup_qserver( server, 1);
-		return;
+		return PKT_ERROR;
 	}
 
 	while ( 1 == type && ptr < end )
@@ -188,8 +189,7 @@ void deal_with_gs2_packet( struct qserver *server, char *rawpkt, int pktlen )
 			{
 				free( headers );
 			}
-			cleanup_qserver( server, 1);
-			return;
+			return MEM_ERROR;
 		}
 
 		headers = tmpp;
@@ -211,8 +211,7 @@ void deal_with_gs2_packet( struct qserver *server, char *rawpkt, int pktlen )
 		// no more info should be player info here as we
 		// requested it
 		malformed_packet( server, "no players" );
-		cleanup_qserver( server, 1);
-		return;
+		return PKT_ERROR;
 	}
 
 	while( 2 == type && ptr < end )
@@ -225,8 +224,7 @@ void deal_with_gs2_packet( struct qserver *server, char *rawpkt, int pktlen )
 			if ( 0 != no_players )
 			{
 				malformed_packet( server, "no players" );
-				cleanup_qserver( server, 1);
-				return;
+				return PKT_ERROR;
 			}
 		}
 		else
@@ -242,8 +240,7 @@ void deal_with_gs2_packet( struct qserver *server, char *rawpkt, int pktlen )
 				if ( ptr >= end )
 				{
 					malformed_packet( server, "short player detail" );
-					cleanup_qserver( server, 1);
-					return;
+					return PKT_ERROR;
 				}
 				val = ptr;
 				val_len = strlen( val );
@@ -292,8 +289,7 @@ void deal_with_gs2_packet( struct qserver *server, char *rawpkt, int pktlen )
 		if ( total_players > no_players )
 		{
 			malformed_packet( server, "to many players %d > %d", total_players, no_players );
-			cleanup_qserver( server, 1);
-			return;
+			return PKT_ERROR;
 		}
 
 		// check for end of player info
@@ -302,8 +298,7 @@ void deal_with_gs2_packet( struct qserver *server, char *rawpkt, int pktlen )
 			if ( total_players != no_players )
 			{
 				malformed_packet( server, "bad number of players %d != %d", total_players, no_players );
-				cleanup_qserver( server, 1);
-				return;
+				return PKT_ERROR;
 			}
 			type = 3;
 			ptr++;
@@ -315,8 +310,7 @@ void deal_with_gs2_packet( struct qserver *server, char *rawpkt, int pktlen )
 		// no more info should be team info here as we
 		// requested it
 		malformed_packet( server, "no teams" );
-		cleanup_qserver( server, 1 );
-		return;
+		return PKT_ERROR;
 	}
 
 	no_teams = (unsigned char)*ptr;
@@ -340,8 +334,7 @@ void deal_with_gs2_packet( struct qserver *server, char *rawpkt, int pktlen )
 			{
 				free( headers );
 			}
-			cleanup_qserver( server, 1);
-			return;
+			return MEM_ERROR;
 		}
 
 		headers = tmpp;
@@ -362,8 +355,7 @@ void deal_with_gs2_packet( struct qserver *server, char *rawpkt, int pktlen )
 		// no more info should be team info here as we
 		// requested it
 		malformed_packet( server, "no teams" );
-		cleanup_qserver( server, 1);
-		return;
+		return PKT_ERROR;
 	}
 
 	while( 4 == type && ptr < end )
@@ -378,8 +370,7 @@ void deal_with_gs2_packet( struct qserver *server, char *rawpkt, int pktlen )
 			if ( ptr >= end )
 			{
 				malformed_packet( server, "short team detail" );
-				cleanup_qserver( server, 1);
-				return;
+				return PKT_ERROR;
 			}
 			val = ptr;
 			val_len = strlen( val );
@@ -398,11 +389,9 @@ void deal_with_gs2_packet( struct qserver *server, char *rawpkt, int pktlen )
 		if ( total_teams > no_teams )
 		{
 			malformed_packet( server, "to many teams" );
-			cleanup_qserver( server, 1);
-			return;
+			return PKT_ERROR;
 		}
 	}
 
-	cleanup_qserver( server, 1);
-	return;
+	return DONE_FORCE;
 }
