@@ -1,5 +1,5 @@
 /*
- * qstat 2.8
+ * qstat 2.14
  * by Steve Jankowski
  *
  * New Gamespy v3 query protocol
@@ -51,7 +51,7 @@ int process_gs3_packet( struct qserver *server );
 //	query: [0xFE][0xFD][0x00][0x.. 4-byte-instance][0xb3412b5a "-1287574694"]
 //
 
-void deal_with_gs3_packet( struct qserver *server, char *rawpkt, int pktlen )
+query_status_t deal_with_gs3_packet( struct qserver *server, char *rawpkt, int pktlen )
 {
 	char *ptr = rawpkt;
 	unsigned int pkt_id;
@@ -59,14 +59,48 @@ void deal_with_gs3_packet( struct qserver *server, char *rawpkt, int pktlen )
 	unsigned char flag;
 	unsigned int pkti, final;
 
-	debug( 2, "packet..." );
+	debug( 2, "packet n_requests %d, retry1 %d, n_retries %d, delta %d", server->n_requests, server->retry1, n_retries, time_delta( &packet_recv_time, &server->packet_time1 ) );
+	server->ping_total += time_delta( &packet_recv_time, &server->packet_time1 );
+
+	if ( 7 == pktlen && 0x09 == *ptr )
+	{
+		// gs4 query sent to a gs3 server
+		// switch protocols and try again
+
+		// Correct the stats due to two phase protocol
+		int i;
+		server_type *gs3_type = &builtin_types[50];
+		debug( 3, "Attempting gs3 fallback from gs4" );
+		server->retry1++;
+		if ( GAMESPY3_PROTOCOL_SERVER != gs3_type->id )
+		{
+			// Static coding failure do it the hard way
+			debug( 1, "gs3 static lookup failure, using dynamic lookup" );
+			for( i = 0; i < GAMESPY3_PROTOCOL_SERVER; i++ )
+			{
+				if ( GAMESPY3_PROTOCOL_SERVER == builtin_types[i].id )
+				{
+					// found it
+					gs3_type = &builtin_types[i];
+					i = GAMESPY3_PROTOCOL_SERVER;
+				}
+			}
+
+			if ( GAMESPY3_PROTOCOL_SERVER != gs3_type->id )
+			{
+				malformed_packet( server, "GS3 protocol not found" );
+				return PKT_ERROR;
+			}
+		}
+		server->type = gs3_type;
+		return send_gs3_request_packet( server );
+	}
 
 	if ( pktlen < 12 )
 	{
 		// invalid packet?
 		malformed_packet( server, "too short" );
-		cleanup_qserver( server, 1 );
-		return;
+		return PKT_ERROR;
 	}
 
 	if ( 0x09 == *ptr )
@@ -78,39 +112,20 @@ void deal_with_gs3_packet( struct qserver *server, char *rawpkt, int pktlen )
 		memcpy( &pkt_id, ptr, 4 );
 		ptr += 4;
 		server->challenge = atoi( ptr );
-
-		// Correct the stats due to two phase protocol
+		debug( 3, "Challenge: %ld", server->challenge );
 		server->retry1++;
-		server->n_packets--;
-		if ( server->retry1 == n_retries || server->flags & FLAG_BROADCAST )
-		{
-			server->n_requests--;
-		}
-		else
-		{
-			server->n_retries--;
-		}
-		send_gs3_request_packet( server );
-		return;
+
+		return send_gs3_request_packet( server );
 	}
 
 	if ( 0x00 != *ptr )
 	{
 		malformed_packet( server, "bad initial byte '%hhx'", *ptr );
-		cleanup_qserver( server, 1 );
-		return;
+		return PKT_ERROR;
 	}
 	ptr++;
 
 	server->n_servers++;
-	if ( server->server_name == NULL )
-	{
-		server->ping_total += time_delta( &packet_recv_time, &server->packet_time1 );
-	}
-	else
-	{
-		gettimeofday( &server->packet_time1, NULL);
-	}
 
 	// Could check the header here should
 	// match the 4 byte id sent
@@ -123,14 +138,12 @@ void deal_with_gs3_packet( struct qserver *server, char *rawpkt, int pktlen )
 		if ( server->flags & TF_STATUS_QUERY )
 		{
 			// we have the status response
-			deal_with_gs3_status( server, ptr, pktlen - ( ptr - rawpkt )  );
-			return;
+			return deal_with_gs3_status( server, ptr, pktlen - ( ptr - rawpkt )  );
 		}
 		else
 		{
 			malformed_packet( server, "missing splitnum" );
-			cleanup_qserver( server, 1 );
-			return;
+			return PKT_ERROR;
 		}
 	}
 	ptr += 9;
@@ -167,20 +180,18 @@ void deal_with_gs3_packet( struct qserver *server, char *rawpkt, int pktlen )
 		if ( ! add_packet( server, pkt_id, pkt_index, pkt_max, pktlen, rawpkt, 1 ) )
 		{
 			// fatal error e.g. out of memory
-			return;
+			return MEM_ERROR;
 		}
 
 		// combine_packets will call us recursively
-		combine_packets( server );
-		return;
+		return combine_packets( server );
 	}
 
 	// if we get here we have what should be a full packet
-	process_gs3_packet( server );
-	return;
+	return process_gs3_packet( server );
 }
 
-void deal_with_gs3_status( struct qserver *server, char *rawpkt, int pktlen )
+query_status_t deal_with_gs3_status( struct qserver *server, char *rawpkt, int pktlen )
 {
 	char *pkt = rawpkt;
 	debug( 1, "status packet" );
@@ -194,8 +205,90 @@ void deal_with_gs3_status( struct qserver *server, char *rawpkt, int pktlen )
 	pkt += strlen( pkt ) + 1;
 
 	// map
-	server->map_name = strdup( pkt );
-	pkt += strlen( pkt ) + 1;
+	// UT3 retail compatibility
+	if ( 0 == strncmp( pkt, "OwningPlayerId", 13 ) )
+	{
+		char *end = pkt + strlen( pkt ) + 1;
+		char *var = pkt;
+		while ( NULL != var )
+		{
+			char *next;
+			char *val = strstr( var, "=" );
+			*val = '\0';
+			val++;
+			next = strstr( val, "," );
+			if ( NULL != next )
+			{
+				*next = '\0';
+				next++;
+			}
+
+			if ( 0 == strcmp( var, "mapname" ) )
+			{
+				if ( server->map_name )
+				{
+					free( server->map_name );
+				}
+				server->map_name = strdup( val );
+			}
+			else if ( 0 == strcmp( var, "p1073741825" ) )
+			{
+				if ( server->map_name )
+				{
+					free( server->map_name );
+				}
+				server->map_name = strdup( val );
+			}
+			else if ( 0 == strcmp( var, "p1073741826" ) )
+			{
+				add_rule( server, "gametype", val, OVERWITE_DUPLICATES );
+			}
+			else if ( 0 == strcmp( var, "p268435705" ) )
+			{
+				add_rule( server, "timelimit", val, OVERWITE_DUPLICATES );
+			}
+			else if ( 0 == strcmp( var, "p1073741827" ) )
+			{
+				add_rule( server, "description", val, OVERWITE_DUPLICATES );
+#ifndef UT3_PATCHED
+				if ( 0 != strlen( val ) )
+				{
+					if ( server->server_name )
+					{
+						char *name = (char*)realloc( server->server_name, strlen( server->server_name ) + strlen( val ) + 3 );
+						if ( name )
+						{
+							strcat( name, ": " );
+							strcat( name, val );
+							server->server_name = name;
+						}
+					}
+					server->server_name = strdup( val );
+				}
+#endif
+			}
+			else if ( 0 == strcmp( var, "p268435704" ) )
+			{
+				add_rule( server, "goalscore", val, OVERWITE_DUPLICATES );
+			}
+			else if ( 0 == strcmp( var, "s32779" ) )
+			{
+				add_rule( server, "gamemode", val, OVERWITE_DUPLICATES );
+			}
+			else
+			{
+				add_rule( server, var, val, OVERWITE_DUPLICATES );
+			}
+
+			var = next;
+		}
+		pkt = end;
+	}
+	else
+	{
+		server->map_name = strdup( pkt );
+		pkt += strlen( pkt ) + 1;
+	}
 
 	// num players
 	server->num_players = atoi( pkt );
@@ -209,7 +302,7 @@ void deal_with_gs3_status( struct qserver *server, char *rawpkt, int pktlen )
 	change_server_port( server, atoi( pkt ), 0 );
 	pkt += strlen( pkt ) + 1;
 
-	cleanup_qserver( server, 1 );
+	return DONE_FORCE;
 }
 
 int process_gs3_packet( struct qserver *server )
@@ -231,13 +324,17 @@ int process_gs3_packet( struct qserver *server )
 		char *end = ptr + pktlen;
 		debug( 2, "processing fragment[%d]...", fragment->pkt_index );
 
+		if( 6 <= get_debug_level() )
+		{
+			print_packet( server, ptr, pktlen );
+		}
+
 		// check we have a full header
 		if ( pktlen < 16 )
 		{
 			// invalid packet?
 			malformed_packet( server, "too short" );
-			cleanup_qserver( server, 1 );
-			return 0;
+			return PKT_ERROR;
 		}
 
 		// skip over the header
@@ -251,7 +348,15 @@ int process_gs3_packet( struct qserver *server )
 			char *var, *val;
 			int var_len, val_len;
 
-			if ( 0x00 == ptr[0] && 0x01 == ptr[1] )
+			if ( ptr + 1 >= end )
+			{
+				debug( 4, "state = %d, %hhx, bytes left = %d", state, ptr[0], (int)(end - ptr) );
+			}
+			else
+			{
+				debug( 4, "state = %d, %hhx, %hhx, bytes left = %d", state, ptr[0], ptr[1], (int)(end - ptr) );
+			}
+			if ( 0x00 == ptr[0] && ( ptr + 1 >= end || 0x01 >= ptr[1] ) )
 			{
 				// not quite sure of the significance of these bytes
 				// but we use them as a check for end of section
@@ -267,8 +372,7 @@ int process_gs3_packet( struct qserver *server )
 			if ( ptr + 1 > end )
 			{
 				malformed_packet( server, "no rule value" );
-				cleanup_qserver( server, 1);
-				return 0;
+				return PKT_ERROR;
 			}
 
 			val = ptr;
@@ -294,6 +398,89 @@ int process_gs3_packet( struct qserver *server )
 			}
 			else if( 0 == strcmp( var, "mapname" ) )
 			{
+				// UT3 retail compatibility
+				if ( 0 == strncmp( val, "OwningPlayerId", 13 ) )
+				{
+					var = val;
+					while ( NULL != var )
+					{
+						char *next;
+						char *val = strstr( var, "=" );
+						*val = '\0';
+						val++;
+						next = strstr( val, "," );
+						if ( NULL != next )
+						{
+							*next = '\0';
+							next++;
+						}
+
+						if ( 0 == strcmp( var, "mapname" ) )
+						{
+							if ( server->map_name )
+							{
+								free( server->map_name );
+							}
+							server->map_name = strdup( val );
+						}
+						else if ( 0 == strcmp( var, "p1073741825" ) )
+						{
+							if ( server->map_name )
+							{
+								free( server->map_name );
+							}
+							server->map_name = strdup( val );
+						}
+						else if ( 0 == strcmp( var, "p1073741826" ) )
+						{
+							add_rule( server, "gametype", val, OVERWITE_DUPLICATES );
+						}
+						else if ( 0 == strcmp( var, "p268435705" ) )
+						{
+							add_rule( server, "timelimit", val, OVERWITE_DUPLICATES );
+						}
+						else if ( 0 == strcmp( var, "p1073741827" ) )
+						{
+#ifndef UT3_PATCHED
+							if ( 0 != strlen( val ) )
+							{
+								if ( server->server_name )
+								{
+									char *name = (char*)realloc( server->server_name, strlen( server->server_name ) + strlen( val ) + 3 );
+									if ( name )
+									{
+										strcat( name, ": " );
+										strcat( name, val );
+										server->server_name = name;
+									}
+								}
+								server->server_name = strdup( val );
+							}
+#endif
+						}
+						else if ( 0 == strcmp( var, "p268435704" ) )
+						{
+							add_rule( server, "goalscore", val, OVERWITE_DUPLICATES );
+						}
+						else if ( 0 == strcmp( var, "s32779" ) )
+						{
+							add_rule( server, "gamemode", val, OVERWITE_DUPLICATES );
+						}
+						else
+						{
+							add_rule( server, var, val, OVERWITE_DUPLICATES );
+						}
+						var = next;
+					}
+				}
+				else
+				{
+					server->map_name = strdup( val );
+				}
+			}
+			else if( 0 == strcmp( var, "map" ) )
+			{
+				// BF2MC compatibility
 				server->map_name = strdup( val );
 			}
 			else if( 0 == strcmp( var, "maxplayers" ) )
@@ -308,43 +495,98 @@ int process_gs3_packet( struct qserver *server )
 			else if( 0 == strcmp( var, "hostport" ) )
 			{
 				change_server_port( server, atoi( val ), 0 );
-				add_rule( server, var, val, NO_FLAGS );
+			}
+			else if ( 0 == strcmp( var, "p1073741825" ) )
+			{
+				// UT3 demo compatibility
+				if ( server->map_name )
+				{
+					free( server->map_name );
+				}
+				server->map_name = strdup( val );
+			}
+			else if ( 0 == strcmp( var, "p1073741826" ) )
+			{
+				// UT3 demo compatibility
+				add_rule( server, "gametype", val, OVERWITE_DUPLICATES );
+			}
+			else if ( 0 == strcmp( var, "p268435705" ) )
+			{
+				// UT3 demo compatibility
+				add_rule( server, "timelimit", val, OVERWITE_DUPLICATES );
+			}
+			else if ( 0 == strcmp( var, "p1073741827" ) )
+			{
+				// UT3 demo compatibility
+				add_rule( server, "description", val, OVERWITE_DUPLICATES );
+#ifndef UT3_PATCHED
+				if ( 0 != strlen( val ) )
+				{
+					if ( server->server_name )
+					{
+						char *name = (char*)realloc( server->server_name, strlen( server->server_name ) + strlen( val ) + 3 );
+						if ( name )
+						{
+							strcat( name, ": " );
+							strcat( name, val );
+							server->server_name = name;
+						}
+					}
+					server->server_name = strdup( val );
+				}
+#endif
+			}
+			else if ( 0 == strcmp( var, "p268435704" ) )
+			{
+				// UT3 demo compatibility
+				add_rule( server, "goalscore", val, OVERWITE_DUPLICATES );
+			}
+			else if ( 0 == strcmp( var, "s32779" ) )
+			{
+				// UT3 demo compatibility
+				add_rule( server, "gamemode", val, OVERWITE_DUPLICATES );
 			}
 			else
 			{
-				add_rule( server, var, val, NO_FLAGS );
+				add_rule( server, var, val, OVERWITE_DUPLICATES );
 			}
 		}
 
+
 		while ( 1 == state && ptr < end )
 		{
 			// first we have the header
 			char *header = ptr;
 			int head_len = strlen( header );
 			int header_type;
-			ptr += head_len + 1;
+
+			debug( 4, "state = %d, bytes left = %d, head_len = %d", state, (int)(end - ptr), head_len );
 
 			if ( 0 == head_len )
 			{
 				// no more info
 				debug( 3, "All done" );
-				cleanup_qserver( server, 1 );
-				return 1;
+				return DONE_FORCE;
+			}
+
+			ptr += head_len + 1;
+
+			if ( ptr >= end )
+			{
+				// partial header, should be restarted in the next fragment
+				debug( 5, "partial header '%s'", header );
+				// ensure gt than
+				ptr++;
+
+				break;
 			}
 
 			debug( 2, "player header '%s'", header );
 
-			if ( ptr > end )
-			{
-				malformed_packet( server, "no details for header '%s'", header );
-				cleanup_qserver( server, 1 );
-				return 0;
-			}
-
 			// the next byte is the starting number
 			total_players = *ptr++;
 
-			if ( 0 == strcmp( header, "player_" ) )
+			if ( 0 == strcmp( header, "player_" ) || 0 == strcmp( header, "name_" ) )
 			{
 				header_type = PLAYER_NAME_HEADER;
 			}
@@ -384,7 +626,7 @@ int process_gs3_packet( struct qserver *server )
 				// check for end of this headers player info
 				if ( 0x00 == *ptr )
 				{
-					debug( 3, "end of '%s' detail", header );
+					debug( 3, "end of '%s' detail, %d bytes left", header, (int)(end - ptr) );
 					ptr++;
 					// Note: can't check ( total_players != no_players ) here as we may have more packets
 					if ( ptr < end && 0x00 == *ptr )
@@ -406,8 +648,7 @@ int process_gs3_packet( struct qserver *server )
 				if ( ptr >= end )
 				{
 					malformed_packet( server, "short player detail" );
-					cleanup_qserver( server, 1);
-					return 0;
+					return PKT_ERROR;
 				}
 				val = ptr;
 				val_len = strlen( val );
@@ -462,13 +703,14 @@ int process_gs3_packet( struct qserver *server )
 				if ( total_players > no_players )
 				{
 					malformed_packet( server, "to many players %d > %d", total_players, no_players );
-					cleanup_qserver( server, 1 );
-					return 0;
+					return PKT_ERROR;
 				}
 			}
 		}
 
-		if ( 2 == state )
+		debug( 4, "state = %d, bytes left = %d", state, (int)(end - ptr) );
+
+		if ( 2 == state && ptr < end )
 		{
 			no_teams = (unsigned char)*ptr;
 			ptr++;
@@ -489,8 +731,14 @@ int process_gs3_packet( struct qserver *server )
 			{
 				// no more info
 				debug( 3, "All done" );
-				cleanup_qserver( server, 1 );
-				return 1;
+				return DONE_FORCE;
+			}
+
+			if ( ptr >= end )
+			{
+				// partial header, should be restarted in the next fragment
+				debug( 5, "partial header '%s'", header );
+				break;
 			}
 
 			debug( 2, "team header '%s'", header );
@@ -516,8 +764,7 @@ int process_gs3_packet( struct qserver *server )
 				if ( ptr >= end )
 				{
 					malformed_packet( server, "short team detail" );
-					cleanup_qserver( server, 1);
-					return 0;
+					return PKT_ERROR;
 				}
 				val = ptr;
 				val_len = strlen( val );
@@ -552,11 +799,10 @@ int process_gs3_packet( struct qserver *server )
 		}
 	}
 
-	cleanup_qserver( server, 1 );
-	return 1;
+	return DONE_FORCE;
 }
 
-void send_gs3_request_packet( struct qserver *server )
+query_status_t send_gs3_request_packet( struct qserver *server )
 {
 	char *packet;
 	char query_buf[128];
@@ -615,5 +861,5 @@ void send_gs3_request_packet( struct qserver *server )
 		}
 	}
 
-	send_packet( server, packet, len );
+	return send_packet( server, packet, len );
 }
