@@ -1,5 +1,5 @@
 /*
- * qstat 2.8
+ * qstat 2.14
  * by Steve Jankowski
  *
  * New Gamespy v3 query protocol
@@ -13,8 +13,8 @@
 #include "qserver.h"
 
 // Packet processing methods
-void deal_with_gs3_packet( struct qserver *server, char *pkt, int pktlen );
-void deal_with_gs3_status( struct qserver *server, char *rawpkt, int pktlen );
-void send_gs3_request_packet( struct qserver *server );
+query_status_t deal_with_gs3_packet( struct qserver *server, char *pkt, int pktlen );
+query_status_t deal_with_gs3_status( struct qserver *server, char *rawpkt, int pktlen );
+query_status_t send_gs3_request_packet( struct qserver *server );
 
 #endif
