@@ -1,5 +1,5 @@
 /*
- * qstat 2.8
+ * qstat 2.14
  * by Steve Jankowski
  *
  * Packet module
@@ -15,7 +15,7 @@
 #include <stdio.h>
 
 #define MAX_PACKETS 8
-#define MAX_FAGMENTS 16
+#define MAX_FAGMENTS 128
 
 int pkt_seq = 0;
 int pkt_id_index = -1;
@@ -29,7 +29,7 @@ int combine_packets( struct qserver *server )
 	int maxes[MAX_PACKETS];
 	int lengths[MAX_PACKETS];
 	SavedData *sdata = &server->saved_data;
-	int i, p, done = 0;
+	int i, p, ret = INPROGRESS;
 	n_ids = 0;
 
 	memset( &segments[0][0], 0, sizeof(segments) );
@@ -46,6 +46,13 @@ int combine_packets( struct qserver *server )
 			continue;
 		}
 
+		if ( sdata->pkt_index >= MAX_FAGMENTS )
+		{
+			// we only deal up to MAX_FAGMENTS packet fragment
+			fprintf( stderr, "Too many fragments %d for packetid %d max %d\n", sdata->pkt_index, sdata->pkt_id, MAX_FAGMENTS );
+			return PKT_ERROR;
+		}
+
 		for ( i = 0; i < n_ids; i++ )
 		{
 			if ( sdata->pkt_id == ids[i])
@@ -62,7 +69,7 @@ int combine_packets( struct qserver *server )
 			{
 				// we only deal up to MAX_PACKETS packetids
 				fprintf( stderr, "Too many distinct packetids %d max %d\n", n_ids, MAX_PACKETS );
-				continue;
+				return PKT_ERROR;
 			}
 			ids[n_ids]= sdata->pkt_id;
 			maxes[n_ids]= sdata->pkt_max;
@@ -78,12 +85,6 @@ int combine_packets( struct qserver *server )
 		if ( segments[i][sdata->pkt_index] == NULL)
 		{
 			// add the packet to the list of segments
-			if ( sdata->pkt_index >= MAX_FAGMENTS )
-			{
-				// we only deal up to MAX_FAGMENTS packet fragment
-				fprintf( stderr, "Too many fragments for packetid %d max %d\n", sdata->pkt_id, MAX_FAGMENTS );
-				continue;
-			}
 			segments[i][sdata->pkt_index]= sdata;
 			counts[i]++;
 			lengths[i] += sdata->datalen;
@@ -98,7 +99,9 @@ int combine_packets( struct qserver *server )
 	for ( pkt_id_index = 0; pkt_id_index < n_ids; pkt_id_index++ )
 	{
 		char *combined;
-		int datalen= 0;
+		int datalen = 0;
+		int combinedlen;
+
 		if ( counts[pkt_id_index] != maxes[pkt_id_index] )
 		{
 			// we dont have all the expected packets yet
@@ -107,7 +110,8 @@ int combine_packets( struct qserver *server )
 		}
 
 		// combine all the segments
-		combined = (char*)malloc( lengths[pkt_id_index] );
+		combinedlen = lengths[pkt_id_index];
+		combined = (char*)malloc(combinedlen);
 		for ( p = 0; p < counts[pkt_id_index]; p++ )
 		{
 			if ( segments[pkt_id_index][p] == NULL )
@@ -116,7 +120,15 @@ int combine_packets( struct qserver *server )
 				// reset to be unusable
 				pkt_id_index = -1;
 				free( combined );
-				return 0;
+				return INPROGRESS;
+			}
+			if (datalen + segments[pkt_id_index][p]->datalen > combinedlen) {
+				fprintf(stderr, "Data length %d > combined length %d\n",
+					datalen + segments[pkt_id_index][p]->datalen, combinedlen);
+				// reset to be unusable
+				pkt_id_index = -1;
+				free(combined);
+				return MEM_ERROR;
 			}
 			memcpy( combined + datalen, segments[pkt_id_index][p]->data, segments[pkt_id_index][p]->datalen );
 			datalen += segments[pkt_id_index][p]->datalen;
@@ -133,13 +145,15 @@ int combine_packets( struct qserver *server )
 		{
 			print_packet( server, combined, datalen );
 		}
-		// Call the server's packet processing method
-		done = ( (int (*)()) server->type->packet_func)( server, combined, datalen );
+		// Call the server's packet processing method flagging as a combine call
+		server->combined = 1;
+		ret = ( (int (*)()) server->type->packet_func)( server, combined, datalen );
 		free( combined );
+		server->combined = 0;
 
 		// Note: this is currently invalid as packet processing methods
 		// are void not int
-		if ( done || server->saved_data.data == NULL)
+		if ( INPROGRESS != ret || NULL == server->saved_data.data )
 		{
 			break;
 		}
@@ -147,21 +161,30 @@ int combine_packets( struct qserver *server )
 	// reset to be unusable
 	pkt_id_index = -1;
 
-	return done;
+	return ret;
 }
 
+// NOTE:
+// pkt_id is the packet aka response identifier
+// pkt_index is the index of the packet fragment
 int add_packet( struct qserver *server, unsigned int pkt_id, int pkt_index, int pkt_max, int datalen, char *data, int calc_max )
 {
 	SavedData *sdata;
 
+	// safety net for bad data
+	if (datalen == 0) {
+		debug(1, "Empty packet received!");
+		return 0;
+	}
+
 	if ( server->saved_data.data == NULL )
 	{
-		debug( 4, "first packet" );
+		debug( 4, "first packet: %d id, %d index, %d max, %d calc_max", pkt_id, pkt_index, pkt_max, calc_max );
 		sdata = &server->saved_data;
 	}
 	else
 	{
-		debug( 4, "another packet" );
+		debug( 4, "another packet: %d id, %d index, %d max, %d calc_max", pkt_id, pkt_index, pkt_max, calc_max );
 		if ( calc_max )
 		{
 			// check we have the correct max
@@ -196,7 +219,6 @@ int add_packet( struct qserver *server, unsigned int pkt_id, int pkt_index, int
 	if ( NULL == sdata->data )
 	{
 		fprintf( stderr, "Out of memory\n" );
-		cleanup_qserver( server, 1 );
 		return 0;
 	}
 
@@ -240,3 +262,19 @@ unsigned combined_length( struct qserver *server, int pkt_id )
 	}
 	return len;
 }
+
+unsigned packet_count( struct qserver *server )
+{
+	SavedData *sdata = &server->saved_data;
+	unsigned cnt = 0;
+	if ( NULL == sdata->data )
+	{
+		return 0;
+	}
+
+	for ( ; sdata != NULL; sdata = sdata->next )
+	{
+		cnt++;
+	}
+	return cnt;
+}
