@@ -1,5 +1,5 @@
 /*
- * qstat 2.6
+ * qstat 2.14
  * by Steve Jankowski
  *
  * qserver functions and definitions
@@ -27,6 +27,17 @@ typedef struct SavedData  {
     struct SavedData *next;
 } SavedData;
 
+typedef enum {
+    STATE_INIT			=  0,
+    STATE_CONNECTING	=  1,
+    STATE_CONNECTED		=  2,
+    STATE_QUERYING		=  3,
+    STATE_QUERIED		=  4,
+
+    STATE_TIMEOUT		= -1,
+    STATE_SYS_ERROR		= -2,
+} server_state_t;
+
 struct qserver {
     char *arg;
     char *host_name;
@@ -34,12 +45,21 @@ struct qserver {
     int flags;
     server_type * type;
     int fd;
+	server_state_t state;
     char *outfilename;
     char *query_arg;
+    char *challenge_string;
     struct query_param *params;
     unsigned long challenge;
     unsigned short port;
+    unsigned short orig_port; // This port is always constant from creation where as port can be updated based on the query results
     unsigned short query_port;
+
+	// State variable to flag if the current processing call is a call from combine_packets
+	// This should really by done via a flag to the method itself but that would require changes
+	// to all handlers :(
+	unsigned short combined;
+
     /** \brief number of retries _left_ for status query or rule query.
      *
      * That means
@@ -103,6 +123,8 @@ struct qserver {
     int max_players;
     int num_players;
     int protocol_version;
+	int max_spectators;
+	int num_spectators;
 
     SavedData saved_data;
 
@@ -166,7 +188,7 @@ unsigned int get_param_ui_value( struct qserver *server, char *key,
  * \param len length of data
  * \returns number of bytes sent or SOCKET_ERROR
  */
-int qserver_send_initial(struct qserver* server, const char* data, size_t len);
+query_status_t qserver_send_initial(struct qserver* server, const char* data, size_t len);
 
 /** \brief send an initial query packet to a server
  *
@@ -176,7 +198,7 @@ int qserver_send_initial(struct qserver* server, const char* data, size_t len);
  *
  * @see qserver_send_initial
  */
-int qserver_send(struct qserver* server, const char* data, size_t len);
+query_status_t qserver_send(struct qserver* server, const char* data, size_t len);
 
 int send_broadcast( struct qserver *server, const char *pkt, size_t pktlen);
 
@@ -185,13 +207,18 @@ int send_broadcast( struct qserver *server, const char *pkt, size_t pktlen);
  *
  * This updates n_requests, n_packets, packet_time1 and decrements n_retries
  */
-void register_send( struct qserver *server );
+int register_send( struct qserver *server );
 
 /**
  * Sends a packet to the server either direct or via broadcast.
  *
  * Once sent calls register_send to register the send of the packet
  */
-int send_packet( struct qserver* server, const char* data, size_t len );
+query_status_t send_packet( struct qserver* server, const char* data, size_t len );
+
+/**
+ * Logs the error from a socket send
+ */
+query_status_t send_error( struct qserver *server, int rc );
 
 #endif
