@@ -1,5 +1,5 @@
 /*
- * qstat 2.8
+ * qstat 2.14
  * by Steve Jankowski
  *
  * Teamspeak 2 protocol
@@ -7,14 +7,14 @@
  *
  * Licensed under the Artistic License, see LICENSE.txt for license terms
  */
-#ifndef QSTAT_GPS_H
-#define QSTAT_GPS_H
+#ifndef QSTAT_TS2_H
+#define QSTAT_TS2_H
 
 #include "qserver.h"
 
 // Packet processing methods
-void deal_with_ts2_packet( struct qserver *server, char *pkt, int pktlen );
-void send_ts2_request_packet( struct qserver *server );
+query_status_t deal_with_ts2_packet( struct qserver *server, char *pkt, int pktlen );
+query_status_t send_ts2_request_packet( struct qserver *server );
 
 #endif
 
