@@ -1,5 +1,5 @@
 /*
- * qstat 2.6
+ * qstat 2.14
  * by Steve Jankowski
  *
  * debug helper functions
@@ -12,7 +12,7 @@
 
 #include "qstat.h"
 
-void send_ut2004master_request_packet(struct qserver *server);
-void deal_with_ut2004master_packet(struct qserver *server, char *rawpkt, int pktlen);
+query_status_t send_ut2004master_request_packet(struct qserver *server);
+query_status_t deal_with_ut2004master_packet(struct qserver *server, char *rawpkt, int pktlen);
 
 #endif
